/* ============================================================
 * File  : tetrahedron.cpp
 * Author: Stephane Gibault
 * Date  : Sun Feb 4 2007
 * Description :
 *
 * 
 *  (C) 2007 by Stephane Gibault
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#include <QtGui>
#include <QtOpenGL>
#include <QFileDialog>
#include "tetrahedron.h"
#include <math.h>

Tetrahedron::Tetrahedron(QWidget *parent)
    : QGLWidget(parent)
{
	setFormat(QGLFormat(QGL::DirectRendering | QGL::DepthBuffer));
    	rotationX = 0.0;
    	rotationY = 0.0;
    	rotationZ = 0.0;
    	deplacementX = 0.0;
    	deplacementY = 0.0;
	zoom = -10.0;
	debut_px = 0;
	debut_py = 0;
	debut_pz = -10;
	debut_rx = 0;
	debut_ry = 0;
	debut_rz = 0;
	//
	fin_px = 0;
	fin_py = 0;
	fin_pz = -10;
	fin_rx = 0;
	fin_ry = 0;
	fin_rz = 0;
	//
	maxanim = 0;
	nameanim = "";
	w_x_p = 2;
	w_x_m = -2;
}

void Tetrahedron::load_effet(int numeffet, QString nomeffet)
{
	glDeleteTextures(1, &texture3 );

	if ( numeffet > 0 )
	{
		iseffect = 1;
		if ( a.load(wg_tet_library + "/BIB_ManSlide/Effects/" + nomeffet + "/" + nomeffet + "000001.jpg") == true )
		{
			nameanim = nomeffet + "/" + nomeffet;
			d = QGLWidget::convertToGLFormat( a );
			glGenTextures( 1, &texture3 );
			glBindTexture( GL_TEXTURE_2D, texture3 );
			glTexImage2D( GL_TEXTURE_2D, 0, 4, d.width(), d.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, d.bits() );
			glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
		}
		else
		{
			qDebug() << "ERROR LOADING EFFECT TEXTURE **********************************";
		}
	}
	else
	{
		iseffect = 0;
    		d = QGLWidget::convertToGLFormat( null_img );
    		glGenTextures( 1, &texture3 );
		glBindTexture( GL_TEXTURE_2D, texture3 );
		glTexImage2D( GL_TEXTURE_2D, 0, 4, d.width(), d.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, d.bits() );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	}
	updateGL();
}

void Tetrahedron::sl_charge_image(QString pathimg, QString pathfond)
{
	glDeleteTextures(1, &texture2 );
	if ( pathimg.right(8) != "$N$U$L$L" )
	{
    		if ( b.load( pathimg ) == true )
		{
			float wratioimg = (float)b.width() / (float)b.height();
			w_x_p = (float)(0-(2 * wratioimg)) * -1;
    			f = QGLWidget::convertToGLFormat( b );
    			glGenTextures( 1, &texture2 );
			glBindTexture( GL_TEXTURE_2D, texture2 );
			glTexImage2D( GL_TEXTURE_2D, 0, 4, f.width(), f.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, f.bits() );
			glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
			w_x_m = w_x_p * -1;
			//
			for(int x=0; x<45; x++)
			{
				// Loop Through The Y Plane
				for(int y=0; y<45; y++)
				{
					// Apply The Wave To Our Mesh
					points[x][y][0]=float((x/5.0f)-4.5f);
					points[x][y][1]=float((y/5.0f)-4.5f);
					points[x][y][2]=float(sin((((x/3.0f)*40.0f)/360.0f)*3.141592654*2.0f));
				}
			}
		}
		else
		{
			qDebug() << "ERROR LOADING TEXTURE PLANE *********************************";
		}
	}
	else
	{
		float wratioimg = (float)null_img.width() / (float)null_img.height();
		w_x_p = (float)(0-(2 * wratioimg)) * -1;
    		f = QGLWidget::convertToGLFormat( null_img );
    		glGenTextures( 1, &texture2 );
		glBindTexture( GL_TEXTURE_2D, texture2 );
		glTexImage2D( GL_TEXTURE_2D, 0, 4, f.width(), f.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, f.bits() );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
		w_x_m = w_x_p * -1;
	}
	//
	if ( pathfond.right(8) != "$N$U$L$L" )
	{
		glDeleteTextures(1, &texture1 );
    		if ( c.load( pathfond ) == true )
		{
    			t = QGLWidget::convertToGLFormat( c );
    			glGenTextures( 1, &texture1 );
			glBindTexture( GL_TEXTURE_2D, texture1 );
			glTexImage2D( GL_TEXTURE_2D, 0, 4, t.width(), t.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, t.bits() );
			glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
		}
		else
		{
			qDebug() << "ERROR LOADING TEXTURE BACKGROUND : " << pathfond;
		}
	}
	else
	{
		glDeleteTextures(1, &texture1 );
		//
    		t = QGLWidget::convertToGLFormat( null_img );
    		glGenTextures( 1, &texture1 );
		glBindTexture( GL_TEXTURE_2D, texture1 );
		glTexImage2D( GL_TEXTURE_2D, 0, 4, t.width(), t.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, t.bits() );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	}
	//
	if ( isfog == 0)
	{
		glDisable(GL_FOG);
	}
	else
	{
		glEnable(GL_FOG);
	}
	updateGL();
}

void Tetrahedron::sl_play()
{
	duree = wg_duree * 25;
	progress = 0;
	progress_anim = 1;
	x_titre = -1.8;
	delta_px = (fin_px - debut_px) / duree;
	delta_py = (fin_py - debut_py) / duree;
	delta_pz = (fin_pz - debut_pz) / duree;
	delta_rx = (fin_rx - debut_rx) / duree;
	delta_ry = (fin_ry - debut_ry) / duree;
	delta_rz = (fin_rz - debut_rz) / duree;
    	rotationX = debut_rx;
    	rotationY = debut_ry;
    	rotationZ = debut_rz;
    	deplacementX = debut_px;
    	deplacementY = debut_py;
	zoom = debut_pz;
	poursuite = 0;
	delta_opacity = (end_opacity - start_opacity) / duree;
	wg_percent = start_opacity;
	QTimer::singleShot(3, this, SLOT(updateMove()));
}

void Tetrahedron::sl_go_start()
{
	rotationX = debut_rx;
	rotationY = debut_ry;
	rotationZ = debut_rz;
	deplacementX = debut_px;
	deplacementY = debut_py;
	zoom = debut_pz;
	x_titre = -1.8;
	updateGL();
}

void Tetrahedron::sl_go_end()
{
	rotationX = fin_rx;
	rotationY = fin_ry;
	rotationZ = fin_rz;
	deplacementX = fin_px;
	deplacementY = fin_py;
	zoom = fin_pz;
	x_titre = -1.8 + ( v_titre * ((wg_duree * 25) + (pause_debut * 25) + (pause_fin * 25)));
	updateGL();
}

void Tetrahedron::updateMove()
{
	if ( progress >= duree + (pause_debut * 25) + (pause_fin * 25) )
	{
		progress = 0;
		wg_percent = start_opacity;
    		rotationX = debut_rx;
    		rotationY = debut_ry;
    		rotationZ = debut_rz;
    		deplacementX = debut_px;
    		deplacementY = debut_py;
		zoom = debut_pz;
		progress_anim = 1;
		x_titre = -1.8;
	}
	else
	{
		if ( progress > (pause_debut * 25) && progress <= (pause_debut * 25) + duree)
		{
    			rotationX = rotationX + delta_rx;
    			rotationY = rotationY + delta_ry;
    			rotationZ = rotationZ + delta_rz;
    			deplacementX = deplacementX + delta_px;
    			deplacementY = deplacementY + delta_py;
			zoom = zoom + delta_pz;
			wg_percent = wg_percent + delta_opacity;
		}
	}
	if ( effet_titre == 3 )
	{
		x_titre = x_titre + v_titre;
	}
	if ( iseffect > 0 )
	{
		if (progress_anim < maxanim)
		{
			progress_anim = progress_anim + 1;
		}
		else
		{
			progress_anim = 1;
		}
		QString	wl_var_1 = "000000";
		QString	wl_var_2 = QString::number(progress_anim);
		wl_var_1.replace( 6 - wl_var_2.length(), wl_var_2.length(), wl_var_2);
		glDeleteTextures( 1, &texture3 );
		if ( a.load(wg_tet_library + "/BIB_ManSlide/Effects/" + nameanim + wl_var_1 + ".jpg") == true )
		{
			d = QGLWidget::convertToGLFormat( a );
		}
		else
		{
			d = QGLWidget::convertToGLFormat( null_img );
		}
		glGenTextures( 1, &texture3 );
		glBindTexture( GL_TEXTURE_2D, texture3 );
		glTexImage2D( GL_TEXTURE_2D, 0, 4, d.width(), d.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, d.bits() );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	}

	if ( poursuite == 0 )
	{
		progress = progress + 1;
		updateGL();
		QTimer::singleShot(30, this, SLOT(updateMove()));
	}
	else
	{
		rotationX = debut_rx;
		rotationY = debut_ry;
		rotationZ = debut_rz;
		deplacementX = debut_px;
		deplacementY = debut_py;
		zoom = debut_pz;
		wg_percent = start_opacity;
		x_titre = -1.8;
		updateGL();
	}
}

void Tetrahedron::sl_stop()
{
	poursuite = 1;
	QTimer::singleShot(50, this, SLOT(sl_wait_stop()));	
}

void Tetrahedron::sl_wait_stop()
{
        QString wl_var_1 = "000001";
	glDeleteTextures(1, &texture3 );
	a.load(wg_tet_library + "/BIB_ManSlide/Effects/" + nameanim + wl_var_1 + ".jpg");
	d = QGLWidget::convertToGLFormat( a );
	glGenTextures( 1, &texture3 );
	glBindTexture( GL_TEXTURE_2D, texture3 );
	glTexImage2D( GL_TEXTURE_2D, 0, 4, d.width(), d.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, d.bits() );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
    	rotationX = debut_rx;
    	rotationY = debut_ry;
    	rotationZ = debut_rz;
    	deplacementX = debut_px;
    	deplacementY = debut_py;
	wg_percent = start_opacity;
	zoom = debut_pz;
	updateGL();
}

void Tetrahedron::sl_mark_deb()
{
	debut_px = deplacementX;
	debut_py = deplacementY;
	debut_pz = zoom;
	debut_rx = rotationX;
	debut_ry = rotationY;
	debut_rz = rotationZ;
}

void Tetrahedron::sl_mark_fin()
{
	fin_px = deplacementX;
	fin_py = deplacementY;
	fin_pz = zoom;
	fin_rx = rotationX;
	fin_ry = rotationY;
	fin_rz = rotationZ;
}

void Tetrahedron::initializeGL()
{
    	qglClearColor(Qt::black);
	//
 	fogcolor[0] = 0.5;
 	fogcolor[1] = 0.5;
 	fogcolor[2] = 0.5;
 	fogcolor[3] = 0.5;
	glDisable(GL_FOG);
 	GLint fogmode = GL_EXP;
 	glFogi (GL_FOG_MODE, fogmode);
 	glFogfv(GL_FOG_COLOR, fogcolor);
 	glFogf(GL_FOG_DENSITY, 0.30);
 	glFogf(GL_FOG_START, 1.0);
 	glFogf(GL_FOG_END, 10.0);
	glEnable( GL_DEPTH_TEST );
	glDisable( GL_NORMALIZE );
	glDisable( GL_DITHER );
	glDisable( GL_SMOOTH );
	glDisable( GL_LIGHTING );
	glEnable(GL_ALPHA_TEST);
	glAlphaFunc(GL_GREATER, 0.0f);

 	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
	//
    	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity(); // *** Renouvellement matrice ***
    	glEnable(GL_TEXTURE_2D);
	if ( null_img.load( QApplication::applicationDirPath() + "/Interface/empty.png" ) == true )
	{
    		t = QGLWidget::convertToGLFormat( null_img );
    		glGenTextures( 1, &texture1 );
		glBindTexture( GL_TEXTURE_2D, texture1 );
		glTexImage2D( GL_TEXTURE_2D, 0, 4, t.width(), t.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, t.bits() );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
    		f = QGLWidget::convertToGLFormat( null_img );
    		glGenTextures( 1, &texture2 );
		glBindTexture( GL_TEXTURE_2D, texture2 );
		glTexImage2D( GL_TEXTURE_2D, 0, 4, f.width(), f.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, f.bits() );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
    		d = QGLWidget::convertToGLFormat( null_img );
    		glGenTextures( 1, &texture3 );
		glBindTexture( GL_TEXTURE_2D, texture3 );
		glTexImage2D( GL_TEXTURE_2D, 0, 4, d.width(), d.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, d.bits() );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	}
	else
	{
		qDebug() << "ERROR WHEN LOADING EMPTY TEXTURE ********************************";
	}
}

void Tetrahedron::change_fog_color()
{
 	GLint fogmode = GL_EXP;
 	glFogfv(GL_FOG_COLOR, fogcolor);
}

void Tetrahedron::sl_opacity(int wopacity)
{
	float recval = (float)wopacity;
	GLfloat fopacity;
	fopacity = recval / 100;
	glFogfv(GL_FOG_DENSITY, &fopacity);
	updateGL();
}

void Tetrahedron::sl_brouillard(int wstate)
{
	isfog = wstate;
	if ( wstate == 0)
	{
		glDisable(GL_FOG);
	}
	else
	{
		glEnable(GL_FOG);
	}
	updateGL();
}

void Tetrahedron::resizeGL(int width, int height)
{
    	glViewport(0, 0, width, height);
    	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
    	GLfloat x = GLfloat(width) / height;
    	glFrustum(-x, x, -1.0, 1.0, 2.0, 20.0);
    	glMatrixMode(GL_MODELVIEW);
}

void Tetrahedron::paintGL()
{
    	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    	draw();
}

void Tetrahedron::sl_change_fond(QString myfic)
{
	if ( myfic.right(8) != "$N$U$L$L" )
	{
		path_fond = myfic;
		glDeleteTextures(1, &texture1 );
		c.load( myfic );
		//
		t = QGLWidget::convertToGLFormat( c );
		glGenTextures( 1, &texture1 );
		glBindTexture( GL_TEXTURE_2D, texture1 );
		glTexImage2D( GL_TEXTURE_2D, 0, 4, t.width(), t.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, t.bits() );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	}
	else
	{
		glDeleteTextures(1, &texture1 );
		t = QGLWidget::convertToGLFormat( null_img );
		glGenTextures( 1, &texture1 );
		glBindTexture( GL_TEXTURE_2D, texture1 );
		glTexImage2D( GL_TEXTURE_2D, 0, 4, t.width(), t.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, t.bits() );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	}
	updateGL();
}

void Tetrahedron::mousePressEvent(QMouseEvent *event)
{
    	lastPos = event->pos();
}

void Tetrahedron::mouseMoveEvent(QMouseEvent *event)
{
    	GLfloat dx = GLfloat(event->x() - lastPos.x()) / width();
    	GLfloat dy = GLfloat(event->y() - lastPos.y()) / height();

    	if (event->buttons() & Qt::LeftButton) 
	{
		if ( tool == 2 )
		{
			deplacementX += (dx * (zoom * -1));
        		deplacementY += (dy * (zoom * -1)) * -1;
		}
		else
		{
			if ( tool == 1 )
			{
				if ( event->x() > lastPos.x() )
				{
					if ( zoom < -2.1 )
					{
						zoom = zoom + 0.1;
					}
				}
				else
				{
					if ( zoom > -16.9 )
					{
						zoom = zoom - 0.1;
					}
				}
			}
			else
			{
				if ( tool == 3 )
				{
					if ( event->x() > lastPos.x() )
					{
						rotationX = rotationX + (1 * (event->x() - lastPos.x()));
					}
					else
					{
						rotationX = rotationX - (1 * (lastPos.x() - event->x()));
					}
				}
				else
				{
					if ( tool == 4 )
					{
						if ( event->x() > lastPos.x() )
						{
							rotationY = rotationY + (1 * (event->x() - lastPos.x()));
						}
						else
						{
							rotationY = rotationY - (1 * (lastPos.x() - event->x()));
						}
					}
					else
					{
						if ( tool == 5 )
						{
							if ( event->x() > lastPos.x() )
							{
								rotationZ = rotationZ + (1 * (event->x() - lastPos.x()));
							}
							else
							{
								rotationZ = rotationZ - (1 * (lastPos.x() - event->x()));
							}
						}
						else
						{
						if ( tool == 6 )
						{
								if ( event->x() > lastPos.x() )
								{
									rotationY = rotationY + (1 * (event->x() - lastPos.x()));
								}
								else
								{
									rotationY = rotationY - (1 * (lastPos.x() - event->x()));
								}
								if ( event->y() > lastPos.y() )
								{
									rotationX = rotationX + (1 * (event->y() - lastPos.y()));
								}
								else
								{
									rotationX = rotationX - (1 * (lastPos.y() - event->y()));
								}
								if ( event->x() > lastPos.x() & event->y() > lastPos.y())
								{
									rotationZ = rotationZ + (1 * (event->x() - lastPos.x()));
								}
								else
								{
									if ( event->x() < lastPos.x() & event->y() < lastPos.y())
									rotationZ = rotationZ - (1 * (lastPos.x() - event->x()));
								}
							}
						}
					}
				}
			} 
		}
        	updateGL();
	}
    	lastPos = event->pos();
}

void Tetrahedron::draw()
{
	glLoadIdentity(); // *** Renouvellement matrice ***
    	glTranslatef(0.0f, 0.0f, -19.9f);
//
    	glBindTexture( GL_TEXTURE_2D, texture1 );
//
    	glBegin(GL_QUADS);
    	glTexCoord2f(1.0f, 0.0f);
    	glVertex3f( 13.0f, -10.0f, 0.0f); 
    	glTexCoord2f(1.0f, 1.0f);
    	glVertex3f(13.0f, 10.0f, 0.0f);
    	glTexCoord2f(0.0f, 1.0f);
    	glVertex3f(-13.0f, 10.0f, 0.0f);
    	glTexCoord2f(0.0f, 0.0f);
	glVertex3f(-13.0f, -10.0f, 0.0f);
    	glEnd();
	//
 	glLoadIdentity(); // *** Renouvellement matrice ***
// *** Transformations ***
	glTranslatef(deplacementX, deplacementY, zoom);
    	glRotatef(rotationX, 1.0, 0.0, 0.0);
    	glRotatef(rotationY, 0.0, 1.0, 0.0);
    	glRotatef(rotationZ, 0.0, 0.0, 1.0);
// *** Gestion de la texture ***
	//
    	glBindTexture( GL_TEXTURE_2D, texture2 );
// *** Creation des elements du dessin ***
	glBegin(GL_QUADS);

	if ( type3D == 0 )
	{
    		glTexCoord2f(1.0f, 0.0f); // A
    		glVertex3f(w_x_p, -2.0f, 0.0f);

    		glTexCoord2f(1.0f, 1.0f); // B
    		glVertex3f(w_x_p, 2.0f, 0.0f);

    		glTexCoord2f(0.0f, 1.0f); // C
    		glVertex3f(w_x_m, 2.0f, 0.0f);

    		glTexCoord2f(0.0f, 0.0f); // D
    		glVertex3f(w_x_m, -2.0f, 0.0f);
	}
	else
	{
		if ( type3D == 1 )
		{
			int x, y; // Loop Variables
			float float_x, float_y, float_xb, float_yb;

			for( x = 0; x < 44; x++ )				// Loop Through The X Plane (44 Points)
			{
				for( y = 0; y < 44; y++ )			// Loop Through The Y Plane (44 Points)
				{
					float_x = float(x)/44;		// Create A Floating Point X Value
					float_y = float(y)/44;		// Create A Floating Point Y Value
					float_xb = float(x+1)/44;		// Create A Floating Point Y Value+0.0227f
					float_yb = float(y+1)/44;		// Create A Floating Point Y Value+0.0227f

					glTexCoord2f( float_x, float_y);	// First Texture Coordinate (Bottom Left)
					glVertex3f( points[x][y][0], points[x][y][1], points[x][y][2] );
			
					glTexCoord2f( float_x, float_yb );	// Second Texture Coordinate (Top Left)
					glVertex3f( points[x][y+1][0], points[x][y+1][1], points[x][y+1][2] );
			
					glTexCoord2f( float_xb, float_yb );	// Third Texture Coordinate (Top Right)
					glVertex3f( points[x+1][y+1][0], points[x+1][y+1][1], points[x+1][y+1][2] );
			
					glTexCoord2f( float_xb, float_y );	// Fourth Texture Coordinate (Bottom Right)
					glVertex3f( points[x+1][y][0], points[x+1][y][1], points[x+1][y][2] );
				}
			}
		}
		else
		{
			if ( type3D == 2 )
			{
				// Front Face
				glNormal3f( 0.0f, 0.0f, 1.0f);					// Normal Pointing Towards Viewer
				glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);	// Point 1 (Front)
				glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);	// Point 2 (Front)
				glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);	// Point 3 (Front)
				glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);	// Point 4 (Front)
				// Back Face
				glNormal3f( 0.0f, 0.0f,-1.0f);					// Normal Pointing Away From Viewer
				glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f);	// Point 1 (Back)
				glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);	// Point 2 (Back)
				glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);	// Point 3 (Back)
				glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);	// Point 4 (Back)
				// Top Face
				glNormal3f( 0.0f, 1.0f, 0.0f);					// Normal Pointing Up
				glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);	// Point 1 (Top)
				glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f,  1.0f,  1.0f);	// Point 2 (Top)
				glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f,  1.0f,  1.0f);	// Point 3 (Top)
				glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);	// Point 4 (Top)
				// Bottom Face
				glNormal3f( 0.0f,-1.0f, 0.0f);					// Normal Pointing Down
				glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f, -1.0f, -1.0f);	// Point 1 (Bottom)
				glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f, -1.0f, -1.0f);	// Point 2 (Bottom)
				glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);	// Point 3 (Bottom)
				glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);	// Point 4 (Bottom)
				// Right face
				glNormal3f( 1.0f, 0.0f, 0.0f);					// Normal Pointing Right
				glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);	// Point 1 (Right)
				glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);	// Point 2 (Right)
				glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);	// Point 3 (Right)
				glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);	// Point 4 (Right)
				// Left Face
				glNormal3f(-1.0f, 0.0f, 0.0f);					// Normal Pointing Left
				glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f);	// Point 1 (Left)
				glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);	// Point 2 (Left)
				glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);	// Point 3 (Left)
				glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);	// Point 4 (Left)
				glEnd();	
			}
		}
	}
	//
    	glEnd();
	//
	if ( iseffect > 0 )
	{
		glLoadIdentity(); // *** Renouvellement matrice ***
		glEnable(GL_BLEND);
		glEnable(GL_DEPTH_TEST);
		//glBlendFunc (GL_SRC_ALPHA, GL_ONE);
		glColor4f(1.0f,1.0f,1.0f,wg_percent);
		glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glBindTexture( GL_TEXTURE_2D, texture3 );
		glTranslatef(0.0, 0.0, -2);
		glBegin(GL_QUADS);
    		glTexCoord2f(1.0f, 0.0f); // A
    		glVertex3f(1.35f, -1.08f, 0.0f);
    		glTexCoord2f(1.0f, 1.0f); // B
    		glVertex3f(1.35f, 1.08f, 0.0f);
    		glTexCoord2f(0.0f, 1.0f); // C
    		glVertex3f(-1.35f, 1.08f, 0.0f);
    		glTexCoord2f(0.0f, 0.0f); // D
    		glVertex3f(-1.35f, -1.08f, 0.0f);
    		glEnd();
		glDisable(GL_BLEND);
		glEnable(GL_DEPTH_TEST);
	}
	//
	if ( txt_sur_image != "" )
	{
		glLoadIdentity();
		//glColor3ub(mycolor.red(),mycolor.green(),mycolor.blue());
		qglColor ( mycolor );
		if ( effet_titre == 0 )
		{
			renderText ( 0, this->height() - 20, txt_sur_image, myfont, 2000 );
		}
		else
		{
			if ( effet_titre == 1 )
			{
				renderText ( 0, 20, txt_sur_image, myfont, 2000 );
			}
			else
			{
				if ( effet_titre == 2 )
				{
					renderText ( 0, this->height() / 2, txt_sur_image, myfont, 2000 );
				}
				else
				{
					if ( effet_titre == 3 )
					{
						renderText ( x_titre, this->height() - 20, txt_sur_image, myfont, 2000 );
					}
				}
			}
		}
		QColor mycoul(255,255,255);
		qglColor ( mycoul );
	}
	//
}

void Tetrahedron::grabframe(QString pathimg)
{
	QImage myimg = this->grabFrameBuffer ( false );
	myimg.save( pathimg ,"JPEG", 80 );
}

void Tetrahedron::grabgl()
{
	myimgrender = this->grabFrameBuffer ( false );
	QString wl_var_1 = "000000000";
	QString wl_var_2 = QString::number(frame_encoding);
	wl_var_1.replace( 9 - wl_var_2.length(), wl_var_2.length(), wl_var_2);
	myimgrender.save(path_render + "/Slide_xxx_" + wl_var_1 + ".jpg","JPEG",wg_qualite);
	frame_encoding = frame_encoding + 1;
	if ( duree < progress )
	{
		poursuite = 1;
	}
}
